@@ -27,7 +27,8 @@ public:
   virtual int minimumHeight() const;
 
   virtual DomElement *createDomElement(bool fitWidth, bool fitHeight,
-				       int& additionalVerticalPadding);
+				       int& additionalVerticalPadding,
+				       WApplication *app);
 
   static bool useJavaScriptHeights(WApplication *app);
 
