@@ -14,6 +14,7 @@ namespace Wt {
 
   class DomElement;
   class StdLayoutImpl;
+  class WApplication;
   class WContainerWidget;
 
 class StdLayoutItemImpl : public WLayoutItemImpl, public WObject
@@ -32,7 +33,8 @@ public:
 
   virtual void containerAddWidgets(WContainerWidget *container) = 0;
   virtual DomElement *createDomElement(bool fitWidth, bool fitHeight,
-				       int& additionalVerticalPadding) = 0;
+				       int& additionalVerticalPadding,
+				       WApplication *app) = 0;
 };
 
 }
