@@ -28,7 +28,8 @@ public:
   virtual WLayoutItem *layoutItem() const;
   virtual void containerAddWidgets(WContainerWidget *container);
   virtual DomElement *createDomElement(bool fitWidth, bool fitHeight,
-				       int& additionalVerticalPadding);
+				       int& additionalVerticalPadding,
+				       WApplication *app);
 
 private:
   WWidgetItem *item_;
