@@ -46,4 +46,10 @@
   #define WT_API
 #endif
 
+#ifdef JAVA
+#define WT_ARRAY volatile
+#else
+#define WT_ARRAY
+#endif
+
 #endif // DLLDEFS_H_
