@@ -8,10 +8,11 @@
 #define WT_EXCEPTION_H_
 
 #include <string>
+#include <Wt/WDllDefs.h>
 
 namespace Wt {
 
-class WtException : public std::exception
+class WT_API WtException : public std::exception
 {
 public:
   WtException(const std::string what);
