@@ -1,11 +1,11 @@
-${APP_CLASS}.private.updateDone = function(updateId) {
+_$_APP_CLASS_$_._p_.updateDone = function(updateId) {
 };
 
-${APP_CLASS}.private.recvCallback = function(request, userData) {
+_$_APP_CLASS_$_._p_.recvCallback = function(request, userData) {
   if (request.readyState == 4) {
     if (request.status) {
       if (request.status == 200) {
-	${APP_CLASS}.private.handleResponse(request.responseText, userData);
+	_$_APP_CLASS_$_._p_.handleResponse(request.responseText, userData);
       } else {
 	//debug("Callback status: " + requests.status);
       }
@@ -17,7 +17,7 @@ ${APP_CLASS}.private.recvCallback = function(request, userData) {
   }
 };
 
-${APP_CLASS}.private.sendUpdate = function(url, data, userData) {
+_$_APP_CLASS_$_._p_.sendUpdate = function(url, data, userData) {
   var xmlHttpReq = false;
   if (window.XMLHttpRequest) {
     xmlHttpReq = new XMLHttpRequest();
@@ -35,9 +35,9 @@ ${APP_CLASS}.private.sendUpdate = function(url, data, userData) {
   xmlHttpReq.open('POST', url, true);
   xmlHttpReq.setRequestHeader("Content-type",
 			      "application/x-www-form-urlencoded;");
-  if (${CLOSE_CONNECTION})
+  if (_$_CLOSE_CONNECTION_$_)
     xmlHttpReq.setRequestHeader("Connection","close");
   xmlHttpReq.onreadystatechange
-  = function() { ${APP_CLASS}.private.recvCallback(xmlHttpReq, userData); };
+  = function() { _$_APP_CLASS_$_._p_.recvCallback(xmlHttpReq, userData); };
   xmlHttpReq.send(data);
-};
\ No newline at end of file
+};
