@@ -1,18 +1,18 @@
-${APP_CLASS}.private.nextUpdateId = 0;
-${APP_CLASS}.private.userData = new Array();
-${APP_CLASS}.private.userCallback = new Array();
+_$_APP_CLASS_$_._p_.nextUpdateId = 0;
+_$_APP_CLASS_$_._p_.userData = new Array();
+_$_APP_CLASS_$_._p_.userCallback = new Array();
 
-${APP_CLASS}.private.updateDone = function(updateId) {
-  var _Wt_ = ${APP_CLASS}.private;
+_$_APP_CLASS_$_._p_.updateDone = function(updateId) {
+  var _Wt_ = _$_APP_CLASS_$_._p_;
   _Wt_.handleResponse("", _Wt_.userData[updateId]);
   _Wt_.userData[updateId] = null;
   var s = document.getElementById("script" + updateId);
   s.parentNode.removeChild(s);
 }
 
-${APP_CLASS}.private.sendUpdate = function(url, data, userdata)
+_$_APP_CLASS_$_._p_.sendUpdate = function(url, data, userdata)
 {
-  var _Wt_ = ${APP_CLASS}.private;
+  var _Wt_ = _$_APP_CLASS_$_._p_;
   var s = document.createElement('SCRIPT');
   _Wt_.userData[_Wt_.nextUpdateId] = userdata;
   s.id = "script" + _Wt_.nextUpdateId;
@@ -20,4 +20,4 @@ ${APP_CLASS}.private.sendUpdate = function(url, data, userdata)
   s.type = 'text/javascript';
   var h = document.getElementsByTagName('HEAD')[0];
   h.appendChild(s);
-}
\ No newline at end of file
+}
