@@ -1,4 +1,4 @@
-var ${WT_CLASS} = {
+var _$_WT_CLASS_$_ = {
 
 DOCUMENT_ELEMENT_NODE: 1,
 DOCUMENT_TEXT_NODE: 3,
@@ -20,21 +20,23 @@ isGecko: navigator.userAgent.toLowerCase().indexOf("gecko") != -1,
 isIEMobile: navigator.userAgent.toLowerCase().indexOf("msie 4") != -1
   || navigator.userAgent.toLowerCase().indexOf("msie 5") != -1,
 
+updateDelay: this.isIE ? 10 : 51,
+
 setHtml: function (el, html) {
   function myImportNode(e, deep) {
+    var newNode, i, il;
     switch (e.nodeType) {
     case 1: // element
-      var newNode;
       if (e.namespaceURI == null)
 	newNode = document.createElement(e.nodeName);
       else
 	newNode = document.createElementNS(e.namespaceURI, e.nodeName);
       if (e.attributes && e.attributes.length > 0)
-	for (var i = 0, il = e.attributes.length; i < il;)
+	for (i = 0, il = e.attributes.length; i < il;)
 	  newNode.setAttribute(e.attributes[i].nodeName,
 			       e.getAttribute(e.attributes[i++].nodeName));
       if (deep && e.childNodes.length > 0)
-	for (var i = 0, il = e.childNodes.length; i < il;)
+	for (i = 0, il = e.childNodes.length; i < il;)
 	  newNode.appendChild(myImportNode(e.childNodes[i++], deep));
       return newNode;
       break;
@@ -46,12 +48,13 @@ setHtml: function (el, html) {
     }
   }
 
-  if (${INNER_HTML}) {
+  if (_$_INNER_HTML_$_) {
     el.innerHTML = html;
   } else {
-    var d = new DOMParser();
-    var b = d.parseFromString('<div>'+html+'<\/div>','application/xhtml+xml');
-    var d = b.documentElement;
+    var d, b;
+    d = new DOMParser();
+    b = d.parseFromString('<div>'+html+'<\/div>','application/xhtml+xml');
+    d = b.documentElement;
     if (d.nodeType != 1) // element
       d = d.nextSibling;
 
@@ -98,25 +101,27 @@ getElement: function(id) {
  * Get coordinates of object/widget relative to page origin.
  */
 widgetPageCoordinates: function(obj) {
-  var objX = objY = 0;
+  var objX = objY = 0, op, lop = null;
 
   while (obj) {
     objX += obj.offsetLeft;
     objY += obj.offsetTop;
-    
-    var op = obj.offsetParent;
+
+    op = obj.offsetParent;
     if (op == null)
       obj = null;
     else {
+      lop = op;
       do {
         obj = obj.parentNode;
-        objX -= obj.scrollLeft;
-        objY -= obj.scrollTop;
+	if (obj.tagName.toUpperCase() === "DIV") {
+	  objX -= obj.scrollLeft;
+	  objY -= obj.scrollTop;
+	}
       } while (obj != op);
     }
   }
 
-  /* we should add document scroll offset ? */
   return { x: objX, y: objY };
 },
 
@@ -124,8 +129,8 @@ widgetPageCoordinates: function(obj) {
  * Get coordinates of (mouse) event relative to a widget.
  */
 widgetCoordinates: function(obj, e) {
-  var p = ${WT_CLASS}.pageCoordinates(e);
-  var w = ${WT_CLASS}.widgetPageCoordinates(obj);
+  var p = _$_WT_CLASS_$_.pageCoordinates(e);
+  var w = _$_WT_CLASS_$_.widgetPageCoordinates(obj);
   return { x: p.x - w.x, y: p.y - w.y };
 },
 
@@ -152,7 +157,7 @@ isKeyPress: function(e) {
 
   var charCode = (typeof e.charCode !== 'undefined') ? e.charCode : 0;
 
-  if (charCode > 0 || ${WT_CLASS}.isIE)
+  if (charCode > 0 || _$_WT_CLASS_$_.isIE)
     return true;
   else
     return (e.keyCode == 13 || e.keyCode == 27 || e.keyCode == 32
@@ -175,6 +180,18 @@ px: function(c, s) {
   return v ? parseInt(v) : 0;
 },
 
+isHidden: function(w) {
+  if (w.style.display == 'none')
+    return true;
+  else { 
+    w = w.parentNode;
+    if (w != null && w.tagName.toLowerCase() != "body")
+      return _$_WT_CLASS_$_.isHidden(w);
+    else
+      return false;
+  }
+},
+
 pxself: function(c, s) {
   var v = c.style[s];
   if (v == 'auto' || v == null)
@@ -186,12 +203,12 @@ pxself: function(c, s) {
 
 IEwidth: function(c, min, max) {
   var r = c.parentNode.clientWidth
-   - ${WT_CLASS}.px(c, 'marginLeft')
-   - ${WT_CLASS}.px(c, 'marginRight')
-   - ${WT_CLASS}.px(c, 'borderLeftWidth')
-   - ${WT_CLASS}.px(c, 'borderRightWidth')
-   - ${WT_CLASS}.px(c.parentNode, 'paddingLeft')
-   - ${WT_CLASS}.px(c.parentNode, 'paddingRight');
+   - _$_WT_CLASS_$_.px(c, 'marginLeft')
+   - _$_WT_CLASS_$_.px(c, 'marginRight')
+   - _$_WT_CLASS_$_.px(c, 'borderLeftWidth')
+   - _$_WT_CLASS_$_.px(c, 'borderRightWidth')
+   - _$_WT_CLASS_$_.px(c.parentNode, 'paddingLeft')
+   - _$_WT_CLASS_$_.px(c.parentNode, 'paddingRight');
 
   var m = /^\s*(\d+)\s*px\s*$/.exec(min);
   var v = m && m.length == 2 ? m[1] : "0";
@@ -212,54 +229,314 @@ IEwidth: function(c, min, max) {
 },
 
 clone: function(o) {
-  for (i in o) {
-    this[i] = o[i];
+  if (o == null || typeof(o) != 'object')
+    return o;
+  var temp = { };
+  for (var i in o) {
+    temp[i] = o[i];
   }
-}
 
+  return temp;
+},
+
+addCss: function(cssText) {
+  var s = document.createElement('style');
+  s.setAttribute('type', 'text/css');
+  s.id = 'main';
+  if (s.styleSheet) { // IE
+    s.styleSheet.cssText = cssText;
+  } else {
+    var t = document.createTextNode(cssText);
+    s.appendChild(t);
+  }
+  var h = document.getElementsByTagName('head')[0];
+  h.appendChild(s);
+},
+
+// from: http://www.hunlock.com/blogs/Totally_Pwn_CSS_with_Javascript
+getCssRule: function(selector, deleteFlag) {
+  selector=selector.toLowerCase();
+
+  if (document.styleSheets) {
+    for (var i=0; i<document.styleSheets.length; i++) {
+      var styleSheet=document.styleSheets[i];
+      var ii=0;
+      var cssRule=false;
+      do {
+	if (styleSheet.cssRules)
+	  cssRule = styleSheet.cssRules[ii];
+	else
+	  cssRule = styleSheet.rules[ii];
+	if (cssRule)
+	  if (cssRule.selectorText.toLowerCase()==selector)
+	    if (deleteFlag=='delete') {
+	      if (styleSheet.cssRules)
+		styleSheet.deleteRule(ii);
+	      else
+		styleSheet.removeRule(ii);
+	      return true;
+	    } else
+	      return cssRule;
+	++ii;
+      } while (cssRule);
+    }
+  }
+
+  return false;
+},
+
+removeCssRule: function(selector) {
+  return _$_WT_CLASS_$_.getCssRule(selector, 'delete');
+},
+
+addStyleSheet: function(uri) {
+  var s = document.createElement('link');
+  s.setAttribute('type', 'text/css');
+  s.setAttribute('href', uri);
+  s.setAttribute('type','text/css');
+  s.setAttribute('rel','stylesheet');
+  var h = document.getElementsByTagName('head')[0];
+  h.appendChild(s);
+},
+
+history: (function() {
+/*
+Original copyright: heavily simplified for Wt
+Copyright (c) 2008, Yahoo! Inc. All rights reserved.
+Code licensed under the BSD License:
+http://developer.yahoo.net/yui/license.txt
+version: 2.5.2
+*/
+  var _UAwebkit = false;
+  var _UAie = false;
+  var _UAopera = false;
+  var _onLoadFn = null;
+  var _histFrame = null;
+  var _stateField = null;
+  var _initialized = false;
+  var _fqstates = [];
+  var _initialState, _currentState;
+  var _onStateChange = function(){};
+  function _getHash() {
+    var i, href;
+    href = location.href;
+    i = href.indexOf("#");
+    return i >= 0 ? href.substr(i + 1) : null;
+  }
+  function _storeStates() {
+    _stateField.value = _initialState + "|" + _currentState;
+    if (_UAwebkit) {
+      _stateField.value += "|" + _fqstates.join(",");
+    }
+  }
+  function _handleFQStateChange(fqstate) {
+    var currentState;
+    if (!fqstate) {
+      _currentState = _initialState;
+      _onStateChange(unescape(_currentState));
+      return;
+    }
+    currentState = fqstate;
+    if (!currentState || _currentState !== currentState) {
+      _currentState = currentState || _initialState;
+      _onStateChange(unescape(_currentState));
+    }
+  }
+  function _updateIFrame (fqstate) {
+    var html, doc;
+    html = '<html><body><div id="state">' + fqstate
+      + '</div></body></html>';
+    try {
+      doc = _histFrame.contentWindow.document;
+      doc.open();
+      doc.write(html);
+      doc.close();
+      return true;
+    } catch (e) {
+      return false;
+    }
+  }
+  function _checkIframeLoaded() {
+    var doc, elem, fqstate, hash;
+    if (!_histFrame.contentWindow || !_histFrame.contentWindow.document) {
+      setTimeout(_checkIframeLoaded, 10);
+      return;
+    }
+    doc = _histFrame.contentWindow.document;
+    elem = doc.getElementById("state");
+    fqstate = elem ? elem.innerText : null;
+    hash = _getHash();
+    setInterval(function () {
+	var newfqstate, newHash;
+	doc = _histFrame.contentWindow.document;
+	elem = doc.getElementById("state");
+	newfqstate = elem ? elem.innerText : null;
+	newHash = _getHash();
+	if (newfqstate !== fqstate) {
+	  fqstate = newfqstate;
+	  _handleFQStateChange(fqstate);
+	  if (!fqstate) {
+	    newHash = _initialState;
+	  } else {
+	    newHash = fqstate;
+	  }
+	  location.hash = newHash;
+	  hash = newHash;
+	  _storeStates();
+	} else if (newHash !== hash) {
+	  hash = newHash;
+	  _updateIFrame(newHash);
+	}
+      }, 50);
+    _initialized = true;
+    if (_onLoadFn != null)
+      _onLoadFn();
+  }
+  function _initialize() {
+    var parts, counter, hash;
+    parts = _stateField.value.split("|");
+    if (parts.length > 1) {
+      _initialState = parts[0];
+      _currentState = parts[1];
+    }
+    if (parts.length > 2) {
+      _fqstates = parts[2].split(",");
+    }
+    if (_UAie) {
+      _checkIframeLoaded();
+    } else {
+      counter = history.length;
+      hash = _getHash();
+      setInterval(function () {
+	  var state, newHash, newCounter;
+	  newHash = _getHash();
+	  newCounter = history.length;
+	  if (newHash !== hash) {
+	    hash = newHash;
+	    counter = newCounter;
+	    _handleFQStateChange(hash);
+	    _storeStates();
+	  } else if (newCounter !== counter && _UAwebkit) {
+	    hash = newHash;
+	    counter = newCounter;
+	    state = _fqstates[counter - 1];
+	    _handleFQStateChange(state);
+	    _storeStates();
+	  }
+	}, 50);
+      _initialized = true;
+      if (_onLoadFn != null)
+	_onLoadFn();
+    }
+  }
+  return {
+  onReady: function (fn) {
+    if (_initialized) {
+      setTimeout(function () { fn(); }, 0);
+    } else {
+      _onLoadFn = fn;
+    }
+  },
+  _initialize: function() {
+    _initialize();
+  },
+  register: function (initialState, onStateChange) {
+    if (_initialized) {
+      throw new Error("All modules must be registered before calling YAHOO.util.History.initialize");
+    }
+    _initialState = escape(initialState);
+    _currentState = _initialState;
+    _onStateChange = onStateChange;
+  },
+  initialize: function (stateField, histFrame) {
+    if (_initialized) {
+      // The browser history manager has already been initialized.
+      return;
+    }
+    var vendor = navigator.vendor || "";
+    if (vendor === "KDE") {
+    } else if (typeof window.opera !== "undefined") {
+      _UAopera = true;
+    } else if (typeof document.all !== "undefined") {
+      _UAie = true;
+    } else if (vendor.indexOf("Apple Computer, Inc.") > -1) {
+      _UAwebkit = true;
+    }
+    if (typeof stateField === "string") {
+      stateField = document.getElementById(stateField);
+    }
+    if (!stateField ||
+	stateField.tagName.toUpperCase() !== "TEXTAREA" &&
+	(stateField.tagName.toUpperCase() !== "INPUT" ||
+	 stateField.type !== "hidden" &&
+	 stateField.type !== "text")) {
+      throw new Error("Missing or invalid argument");
+    }
+    _stateField = stateField;
+    if (_UAie) {
+      if (typeof histFrame === "string") {
+	histFrame = document.getElementById(histFrame);
+      }
+      if (!histFrame || histFrame.tagName.toUpperCase() !== "IFRAME") {
+	throw new Error("Missing or invalid argument");
+      }
+      _histFrame = histFrame;
+    }
+  },
+  navigate: function (state) {
+    if (!_initialized) {
+      //throw new Error("The Browser History Manager is not initialized");
+      return;
+    }
+    fqstate = state;
+    if (_UAie) {
+      return _updateIFrame(fqstate);
+    } else {
+      location.hash = fqstate;
+      if (_UAwebkit) {
+	_fqstates[history.length] = fqstate;
+	_storeStates();
+      }
+      return true;
+    }
+  },
+  getCurrentState: function () {
+    if (!_initialized) {
+      throw new Error("The Browser History Manager is not initialized");
+    }
+    return _currentState;
+  }
+  };
+})()
 };
 
-var ${APP_CLASS} = function() {
+var _$_APP_CLASS_$_ = function() {
 
 var downX = 0;
 var downY = 0;
 
 var saveDownPos = function(e) {
-  var coords = ${WT_CLASS}.pageCoordinates(e);
+  var coords = _$_WT_CLASS_$_.pageCoordinates(e);
   downX = coords.x;
   downY = coords.y;
 };
 
-var historyKey = null;
-var historyStates = ${STATES_INIT};
+var currentHash = null;
 
-var historyChange = function(newLocation, historyData) {
-  if (historyData != null)
-    historyStates = historyData;
-
-  if (historyKey == null) {
-    historyKey = newLocation;
+var onHashChange = function() {
+  var newLocation = _$_WT_CLASS_$_.history.getCurrentState();
+  if (currentHash == newLocation) {
     return;
   } else {
-    historyKey = newLocation;
-    setTimeout("${APP_CLASS}.private.update(null, 'hist', null, true);", 100);
+    currentHash = newLocation;
+    setTimeout("_$_APP_CLASS_$_._p_.update(null, 'hash', null, true);", 1);
   }
 };
 
-var historyChangeState = function(key, value) {
-  if (window.dhtmlHistory) {
-    historyStates[key] = value;
-
-    historyKey = "";
-    for (var key in historyStates) {
-      if (key != "______array") {
-        if (historyKey != "")
-          historyKey += ";"
-        historyKey += key + ":" + historyStates[key];
-      }
-    }
-
-    window.dhtmlHistory.add(historyKey, historyStates);
+var setHash = function(newLocation) {
+  if (currentHash != newLocation) {
+    currentHash = newLocation;
+    _$_WT_CLASS_$_.history.navigate(escape(newLocation));
   }
 };
 
@@ -292,7 +569,7 @@ var dragStart = function(obj, e) {
    *   amts = acceptMimeTypes
    *   ds = dropSignal
    */
-  dragObject = ${WT_CLASS}.getElement(obj.getAttribute("dwid"));
+  dragObject = _$_WT_CLASS_$_.getElement(obj.getAttribute("dwid"));
   dragSourceId = obj.getAttribute("dsid");
   dragObjectPrevStylePosition = dragObject.style["position"];
   dragObjectPrevStyleDisplay = dragObject.style["display"];
@@ -301,11 +578,13 @@ var dragStart = function(obj, e) {
 
   dragObject.style["position"] = 'absolute';
   dragObject.style["display"] = '';
+  dragObject.parentNode.removeChild(dragObject);
+  document.body.appendChild(dragObject);
 
-  var xy = ${WT_CLASS}.pageCoordinates(e);
+  var xy = _$_WT_CLASS_$_.pageCoordinates(e);
 
   if (dragObject == obj) {
-    var widgetXy = ${WT_CLASS}.widgetPageCoordinates(obj);
+    var widgetXy = _$_WT_CLASS_$_.widgetPageCoordinates(obj);
     dragOffsetX = xy.x - widgetXy.x;
     dragOffsetY = xy.y - widgetXy.y;
   } else {
@@ -343,11 +622,8 @@ var dragDrag = function(e) {
     return false;
   }
 
-  var newPosX;
-  var newPosY;
-
   if (dragObject != null) {
-    var xy = ${WT_CLASS}.pageCoordinates(e);
+    var xy = _$_WT_CLASS_$_.pageCoordinates(e);
 
     dragObject.style["left"] = (xy.x - dragOffsetX) + 'px';
     dragObject.style["top"] = (xy.y - dragOffsetY) + 'px';
@@ -355,9 +631,9 @@ var dragDrag = function(e) {
     var prevDropTarget = dragDropTarget;
     dragDropTarget = null;
 
-    for (x = 0; x < dragDropObjects.length; ++x) {
+    for (var x = 0; x < dragDropObjects.length; ++x) {
       theObj = dragDropObjects[x];
-      var objXy = ${WT_CLASS}.widgetPageCoordinates(theObj);
+      var objXy = _$_WT_CLASS_$_.widgetPageCoordinates(theObj);
 
       if ((xy.x > objXy.x) && (xy.x < (objXy.x + theObj.offsetWidth))
 	  && (xy.y > objXy.y) && (xy.y < (objXy.y + theObj.offsetHeight))) {
@@ -387,7 +663,7 @@ var dragDrag = function(e) {
       }
     }
 
-    ${WT_CLASS}.cancelEvent(e);
+    _$_WT_CLASS_$_.cancelEvent(e);
     return false;
   }
 
@@ -419,95 +695,113 @@ var dragEnd = function(e) {
   }
 };
 
-var userEvents = [];
-
-var encodeEvent = function(self, e) {
-  var result = "";
-
-  if (historyKey != null)
-    result += '&historyKey=' + historyKey;
-
-  if (e == userEvents) {
-    for (var i = 0; i < userEvents.length; ++i) {
-      result += '&ue' + i + 'id=' + userEvents[i].id
-        + '&ue' + i + 'name='
-	+ encodeURIComponent(userEvents[i].name)
-        + '&ue' + i + 'an=' + userEvents[i].args.length;
-
-      for (var j = 0; j < userEvents[i].args.length; ++j)
-        result += '&ue' + i + 'a' + j + '='
-         + encodeURIComponent(userEvents[i].args[j]);
-    }
-
-    userEvents = [];
-
-    return result;
-  }
+var encodeEvent = function(self, e, signal, i) {
+  var se = '&e' + i;
+  var result = se + 'signal=' + signal;
 
   if (!e)
     return result;
 
-  result += '&type=' + e.type;
+  if (e.id) {
+    result += se + 'id=' + e.id
+        + se + 'name=' + encodeURIComponent(e.name)
+        + se + 'an=' + e.args.length;
+
+    for (var j = 0; j < e.args.length; ++j)
+      result += se + 'a' + j + '=' + encodeURIComponent(e.args[j]);
+    return result;
+  }
+
+  var t = e.target || e.srcElement;
+  while (!t.id && t.parentNode)
+    t = t.parentNode;
+  if (t.id)
+    result += se + 'tid=' + t.id;
 
   if (e.clientX || e.clientY)
-    result += '&clientX=' + e.clientX + '&clientY=' + e.clientY;
+    result += se + 'clientX=' + e.clientX + se + 'clientY=' + e.clientY;
 
-  var pageCoords = ${WT_CLASS}.pageCoordinates(e);
+  var pageCoords = _$_WT_CLASS_$_.pageCoordinates(e);
   var posX = pageCoords.x;
   var posY = pageCoords.y;
 
   if (posX || posY) {
-    result += '&documentX=' + posX + '&documentY=' + posY;
-    result += '&dragdX=' + (posX - downX)
-      + '&dragdY=' + (posY - downY);
+    result += se + 'documentX=' + posX + se + 'documentY=' + posY;
+    result += se + 'dragdX=' + (posX - downX) + se + 'dragdY=' + (posY - downY);
   }
 
   if (e.screenX || e.screenY)
-    result += '&screenX=' + e.screenX + '&screenY=' + e.screenY;
+    result += se + 'screenX=' + e.screenX + se + 'screenY=' + e.screenY;
 
   if (self) {
-    var widgetCoords = ${WT_CLASS}.widgetPageCoordinates(self);
+    var widgetCoords = _$_WT_CLASS_$_.widgetPageCoordinates(self);
     var objX = widgetCoords.x;
     var objY = widgetCoords.y;
-    result += '&widgetX=' + (posX - objX) + '&widgetY=' + (posY - objY);
+    result += se + 'widgetX=' + (posX - objX) + se + 'widgetY=' + (posY - objY);
   }
 
   if (e.which)
-    result += '&right=' + (e.which==3);
+    result += se + 'right=' + (e.which==3);
   else
     if (e.button)
-      result += '&right=' + (e.button==2);
+      result += se + 'right=' + (e.button==2);
 
   if (typeof e.keyCode !== 'undefined')
-    result += '&keyCode=' + e.keyCode;
+    result += se + 'keyCode=' + e.keyCode;
 
   if (typeof e.charCode !== 'undefined')
-    result += '&charCode=' + e.charCode;
+    result += se + 'charCode=' + e.charCode;
 
   if (e.altKey)
-    result += '&altKey=1';
+    result += se + 'altKey=1';
   if (e.ctrlKey)
-    result += '&ctrlKey=1';
+    result += se + 'ctrlKey=1';
   if (e.metaKey)
-    result += '&metaKey=1';
+    result += se + 'metaKey=1';
   if (e.shiftKey)
-    result += '&shiftKey=1';
+    result += se + 'shiftKey=1';
+
+  if (self)
+    result += se + 'scrollX=' + self.scrollLeft
+      + se + 'scrollY=' + self.scrollTop
+      + se + 'width=' + self.clientWidth
+      + se + 'height=' + self.clientHeight;
 
   return result;
 };
 
-var formObjects = ${FORM_OBJECTS};
-var url = ${RELATIVE_URL};
+var pendingEvents = [];
+
+var encodePendingEvents = function() {
+  var result = "";
+
+  if (currentHash != null)
+    result += '&_=' + encodeURIComponent(unescape(currentHash));
+
+  for (var i = 0; i < pendingEvents.length; ++i) {
+    if (pendingEvents[i].id)
+      result += encodeEvent(null, pendingEvents[i], 'user', i);
+    else
+      result += encodeEvent(pendingEvents[i].object, pendingEvents[i].event,
+			    pendingEvents[i].signal, i);
+  }
+
+  pendingEvents = [];
+
+  return result;
+}
+
+var formObjects = _$_FORM_OBJECTS_$_;
+var url = _$_RELATIVE_URL_$_;
 var quited = false;
 var norestart = false;
 var loaded = false;
 var responsesPending = 0;
 var keepAliveTimer = null;
-var inFrame = ${IN_FRAME};
 
 var doKeepAlive = function() {
   update(null, 'none', null, false);
-  keepAliveTimer = setTimeout(doKeepAlive, ${KEEP_ALIVE}000);
+  keepAliveTimer = setTimeout(doKeepAlive, _$_KEEP_ALIVE_$_000);
 };
 
 var debug = function(s) {
@@ -515,32 +809,30 @@ var debug = function(s) {
 };
 
 var setTitle = function(title) {
-  if (${WT_CLASS}.isIEMobile) return;
-
+  if (_$_WT_CLASS_$_.isIEMobile) return;
   document.title = title;
-  if (inFrame)
-    window.parent.document.title = title;
 };
 
 var load = function() {
+  _$_WT_CLASS_$_.history._initialize();
   if (!loaded) {
     loaded = true;
     setTitle(document.title);
-    ${ONLOAD};
+    _$_ONLOAD_$_;
     update(null, 'load', null, false);
-    keepAliveTimer = setTimeout(doKeepAlive, ${KEEP_ALIVE}000);
+    keepAliveTimer = setTimeout(doKeepAlive, _$_KEEP_ALIVE_$_000);
   }
 };
 
 var cancelFeedback = function(t) {
   clearTimeout(t);
   document.body.style.cursor = 'auto';
-  var div = ${WT_CLASS}.getElement("feedback");
+  var div = _$_WT_CLASS_$_.getElement("feedback");
   if (div) div.style.display = 'none';
 };
 
 var waitFeedback = function() {
-  var div = ${WT_CLASS}.getElement("feedback");
+  var div = _$_WT_CLASS_$_.getElement("feedback");
   if (div)
     div.style.display = '';
   else {
@@ -558,9 +850,9 @@ var handleResponse = function(msg, timer) {
 
   try {
     eval(msg);
-    ${APP_CLASS}.private.autoJavaScript();
+    _$_APP_CLASS_$_._p_.autoJavaScript();
   } catch (e) {
-    alert ("Wt internal error: " + e + ", code: " +  e.code 
+    alert("Wt internal error: " + e + ", code: " +  e.code 
      + ", description: " + e.description /* + ":" + msg */);
   }
 
@@ -568,25 +860,47 @@ var handleResponse = function(msg, timer) {
     cancelFeedback(timer);
 
   --responsesPending;
-  sendUserEvents();
+
+  if (pendingEvents.length > 0)
+    scheduleUpdate(true);
 };
 
 var randomSeed = new Date().getTime();
 
+var updateTimeout = null;
+
 var update = function(self, signalName, e, feedback) {
-  ${APP_CLASS}.private.autoJavaScript();
+  _$_APP_CLASS_$_._p_.autoJavaScript();
+
+  var pendingEvent = new Object();
+  pendingEvent.object = self;
+  pendingEvent.signal = signalName;
+  pendingEvent.event = _$_WT_CLASS_$_.clone(e);
+
+  pendingEvents[pendingEvents.length] = pendingEvent;
+
+  scheduleUpdate(feedback);
+}
+
+var scheduleUpdate = function(feedback) {
+  if (responsesPending == 0) {
+    if (updateTimeout == null)
+      updateTimeout = setTimeout(function() { sendUpdate(feedback); },
+				 _$_WT_CLASS_$_.updateDelay);
+  }
+}
+
+var sendUpdate = function(feedback) {
+  updateTimeout = null;
   ++responsesPending;
 
-  if (${WT_CLASS}.isIEMobile) feedback = false;
+  if (_$_WT_CLASS_$_.isIEMobile) feedback = false;
 
   if (quited) {
     if (norestart)
       return;
     if (confirm("The application was quited, do you want to restart?")) {
-      if (inFrame)
-        window.parent.document.location = window.parent.document.location;
-      else
-        document.location = document.location;
+      document.location = document.location;
       norestart = true;
       return;
     } else {
@@ -597,14 +911,15 @@ var update = function(self, signalName, e, feedback) {
 
   var query = '&rand=' + Math.round(Math.random(randomSeed) * 100000);
 
-  var querydata = 'signal=' + signalName;
-  for (x = 0; x < formObjects.length; ++x) {
-    var el = ${WT_CLASS}.getElement(formObjects[x]);
+  var querydata = encodePendingEvents();
+
+  for (var x = 0; x < formObjects.length; ++x) {
+    var el = _$_WT_CLASS_$_.getElement(formObjects[x]);
     if (el == null)
       continue;
 
     if (el.type == 'select-multiple') {
-      for (i=0; i < el.options.length; i++)
+      for (var i = 0; i < el.options.length; i++)
 	if (el.options[i].selected)
 	  querydata += '&' + formObjects[x]
 	    + '=' + encodeURIComponent(el.options[i].value);
@@ -615,25 +930,17 @@ var update = function(self, signalName, e, feedback) {
 	+ '=' + encodeURIComponent(el.value);
   }
 
-  querydata += encodeEvent(self, e);
+  var tm = feedback ? setTimeout(waitFeedback, _$_INDICATOR_TIMEOUT_$_) : null;
 
-  var tm = feedback ? setTimeout(waitFeedback, ${INDICATOR_TIMEOUT}) : null;
-
-  ${APP_CLASS}.private.sendUpdate(url + query, querydata, tm);
-};
-
-var sendUserEvents = function() {
-  if (responsesPending == 0
-      && userEvents.length > 0) {
-    update(null, "user", userEvents, true);
-  }
+  _$_APP_CLASS_$_._p_.sendUpdate(url + query, querydata, tm);
 };
 
 var emit = function(object, name) {
   var userEvent = new Object();
+
   if (typeof(object) == "string")
     userEvent.id = object;
-  else if (object == ${APP_CLASS})
+  else if (object == _$_APP_CLASS_$_)
     userEvent.id = "app";
   else
     userEvent.id = object.id;
@@ -649,12 +956,14 @@ var emit = function(object, name) {
       userEvent.args[i-2] = arguments[i];
   }
 
-  userEvents[userEvents.length] = userEvent;
-  sendUserEvents();
+  pendingEvents[pendingEvents.length] = userEvent;
+
+  if (responsesPending == 0)
+    scheduleUpdate(true);
 };
 
 var addTimerEvent = function(timerid, msec) {
-  setTimeout('var obj=${WT_CLASS}.getElement("' + timerid + '"); '
+  setTimeout('var obj=_$_WT_CLASS_$_.getElement("' + timerid + '"); '
              + 'if (obj) { obj.onclick(); }', msec);
 };
 
@@ -700,31 +1009,6 @@ var loadScript = function(uri, symbol)
     jsLoaded(uri);
 };
 
-var addStyle = function(style)
-{
-  var s = document.createElement('style');
-  s.setAttribute('type', 'text/css');
-  if (s.styleSheet) { // IE
-    s.styleSheet.cssText = style;
-  } else {
-    var t = document.createTextNode(style);
-    s.appendChild(t);
-  }
-  var h = document.getElementsByTagName('head')[0];
-  h.appendChild(s);
-};
-
-var addStyleSheet = function(uri)
-{
-  var s = document.createElement('link');
-  s.setAttribute('type', 'text/css');
-  s.setAttribute('href', uri);
-  s.setAttribute('type','text/css');
-  s.setAttribute('rel','stylesheet');
-  var h = document.getElementsByTagName('head')[0];
-  h.appendChild(s);
-};
-
 var ImagePreloader = function(uris, callback)
 {
   this.callback = callback;
@@ -757,13 +1041,13 @@ ImagePreloader.prototype.onload = function()
     preloader.callback(preloader.images);
 };
 
+_$_WT_CLASS_$_.history.register('_$_INITIAL_HASH_$_', onHashChange);
+
 /*
  * Public static methods
  */
 return {
-  private: {
-    "addStyle" : addStyle,
-    "addStyleSheet" : addStyleSheet,
+  _p_: {
     "loadScript" : loadScript,
     "onJsLoad" : onJsLoad,
     "setTitle" : setTitle,
@@ -780,11 +1064,11 @@ return {
     "dragEnd" : dragEnd,
     "capture" : capture,
 
-    "historyChangeState" : historyChangeState,
-    "historyChange" : historyChange,
+    "onHashChange" : onHashChange,
+    "setHash" : setHash,
     "ImagePreloader" : ImagePreloader,
 
-    "autoJavaScript" : function() { ${AUTO_JAVASCRIPT} }
+    "autoJavaScript" : function() { _$_AUTO_JAVASCRIPT_$_ }
   },
 
   "emit" : emit
@@ -792,4 +1076,29 @@ return {
 
 }();
 
-var WtSignalEmit = ${APP_CLASS}.emit;
+var WtSignalEmit = _$_APP_CLASS_$_.emit;
+var scriptLoaded = false;
+
+function onLoad() {
+  if (!scriptLoaded) {
+    isLoaded = true;
+    return;
+  }
+
+  _$_WT_CLASS_$_.history.initialize("Wt-history-field", "Wt-history-iframe");
+  window.onresize=_$_APP_CLASS_$_._p_.autoJavaScript;
+  document.body.onmousemove=function(e) {
+    if (!e) e = window.event;
+    return _$_APP_CLASS_$_._p_.dragDrag(e);
+  }
+  document.body.onmouseup=function(e) {
+    if (!e) e = window.event;
+    return _$_APP_CLASS_$_._p_.dragEnd(e);
+  }
+  document.body.ondragstart=function() {
+    return false;
+  };
+  _$_APP_CLASS_$_._p_.load();
+}
+
+function loadWidgetTree() { }
